@@ -15,6 +15,7 @@
 
 from oslo_log import log as logging
 from oslo_utils import strutils
+from oslo_utils import uuidutils
 import pecan
 import six
 
@@ -23,6 +24,7 @@ from zun.api.controllers import link
 from zun.api.controllers.v1 import collection
 from zun.api.controllers.v1.schemas import containers as schema
 from zun.api.controllers.v1.views import containers_view as view
+from zun.api.controllers import versions
 from zun.api import utils as api_utils
 from zun.common import consts
 from zun.common import exception
@@ -31,9 +33,9 @@ from zun.common import name_generator
 from zun.common import policy
 from zun.common import utils
 from zun.common import validation
+from zun.network import neutron
 from zun import objects
 
-
 LOG = logging.getLogger(__name__)
 
 
@@ -51,6 +53,18 @@ def check_policy_on_container(container, action):
     policy.enforce(context, action, container, action=action)
 
 
+def is_all_tenants(search_opts):
+    all_tenants = search_opts.get('all_tenants')
+    if all_tenants:
+        try:
+            all_tenants = strutils.bool_from_string(all_tenants, True)
+        except ValueError as err:
+            raise exception.InvalidInput(six.text_type(err))
+    else:
+        all_tenants = False
+    return all_tenants
+
+
 class ContainerCollection(collection.Collection):
     """API representation of a collection of containers."""
 
@@ -62,6 +76,7 @@ class ContainerCollection(collection.Collection):
     """A list containing containers objects"""
 
     def __init__(self, **kwargs):
+        super(ContainerCollection, self).__init__(**kwargs)
         self._type = 'containers'
 
     @staticmethod
@@ -93,6 +108,9 @@ class ContainersController(base.Controller):
         'top': ['GET'],
         'get_archive': ['GET'],
         'put_archive': ['POST'],
+        'stats': ['GET'],
+        'commit': ['POST'],
+        'add_security_group': ['POST']
     }
 
     @pecan.expose('json')
@@ -108,16 +126,9 @@ class ContainersController(base.Controller):
 
     def _get_containers_collection(self, **kwargs):
         context = pecan.request.context
-        all_tenants = kwargs.get('all_tenants')
-        if all_tenants:
-            try:
-                all_tenants = strutils.bool_from_string(all_tenants, True)
-            except ValueError as err:
-                raise exception.InvalidInput(six.text_type(err))
-        else:
-            # If no value, it's considered to disable all_tenants
-            all_tenants = False
-        if all_tenants:
+        if is_all_tenants(kwargs):
+            policy.enforce(context, "container:get_all_all_tenants",
+                           action="container:get_all_all_tenants")
             context.all_tenants = True
         compute_api = pecan.request.compute_api
         limit = api_utils.validate_limit(kwargs.get('limit'))
@@ -156,36 +167,42 @@ class ContainersController(base.Controller):
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def get_one(self, container_id):
+    def get_one(self, container_id, **kwargs):
         """Retrieve information about the given container.
 
         :param container_ident: UUID or name of a container.
         """
-        container = _get_container(container_id)
-        check_policy_on_container(container.as_dict(), "container:get")
         context = pecan.request.context
+        if is_all_tenants(kwargs):
+            policy.enforce(context, "container:get_one_all_tenants",
+                           action="container:get_one_all_tenants")
+            context.all_tenants = True
+        container = _get_container(container_id)
+        check_policy_on_container(container.as_dict(), "container:get_one")
         compute_api = pecan.request.compute_api
         container = compute_api.container_show(context, container)
         return view.format_container(pecan.request.host_url, container)
 
     def _generate_name_for_container(self):
-        '''Generate a random name like: zeta-22-container.'''
+        """Generate a random name like: zeta-22-container."""
         name_gen = name_generator.NameGenerator()
         name = name_gen.generate()
         return name + '-container'
 
     def _check_for_restart_policy(self, container_dict):
-        '''Check for restart policy input'''
-        name = container_dict.get('restart_policy').get('Name')
-        num = container_dict.get('restart_policy').get('MaximumRetryCount',
-                                                       '0')
+        """Check for restart policy input"""
+        restart_policy = container_dict.get('restart_policy')
+        if not restart_policy:
+            return
+
+        name = restart_policy.get('Name')
+        num = restart_policy.setdefault('MaximumRetryCount', '0')
         count = int(num)
         if name in ['unless-stopped', 'always']:
             if count != 0:
-                raise exception.InvalidValue(("maximum retry "
-                                              "count not valid "
-                                              "with restart policy "
-                                              "of %s") % name)
+                msg = _("maximum retry count not valid with restart "
+                        "policy of %s") % name
+                raise exception.InvalidValue(msg)
         elif name in ['no']:
             container_dict.get('restart_policy')['MaximumRetryCount'] = '0'
 
@@ -198,26 +215,29 @@ class ContainersController(base.Controller):
         """Create a new container.
 
         :param run: if true, starts the container
-        :param container: a container within the request body.
+        :param container_dict: a container within the request body.
         """
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         policy.enforce(context, "container:create",
                        action="container:create")
 
+        # remove duplicate security_groups from list
+        if container_dict.get('security_groups'):
+            container_dict['security_groups'] = list(
+                set(container_dict.get('security_groups')))
         try:
             run = strutils.bool_from_string(run, strict=True)
-        except ValueError:
-            msg = _('Valid run values are true, false, 0, 1, yes and no')
-            raise exception.InvalidValue(msg)
-        try:
             container_dict['interactive'] = strutils.bool_from_string(
                 container_dict.get('interactive', False), strict=True)
         except ValueError:
-            msg = _('Valid interactive value is ''true'', '
+            msg = _('Valid run or interactive value is ''true'', '
                     '"false", True, False, "True" and "False"')
             raise exception.InvalidValue(msg)
 
+        nets = container_dict.get('nets', [])
+        requested_networks = self._build_requested_networks(context, nets)
+
         # Valiadtion accepts 'None' so need to convert it to None
         if container_dict.get('image_driver'):
             container_dict['image_driver'] = api_utils.string_or_none(
@@ -241,20 +261,123 @@ class ContainersController(base.Controller):
                 str(container_dict['memory']) + 'M'
         if container_dict.get('restart_policy'):
             self._check_for_restart_policy(container_dict)
+
+        auto_remove = container_dict.pop('auto_remove', None)
+        if auto_remove is not None:
+            req_version = pecan.request.version
+            min_version = versions.Version('', '', '', '1.3')
+            if req_version >= min_version:
+                try:
+                    container_dict['auto_remove'] = strutils.bool_from_string(
+                        auto_remove, strict=True)
+                except ValueError:
+                    msg = _('Auto_remove value are true or false')
+                    raise exception.InvalidValue(msg)
+            else:
+                msg = _('Invalid param auto_remove because current request '
+                        'version is %(req_version)s. Auto_remove is only '
+                        'supported from version %(min_version)s') % \
+                    {'req_version': req_version,
+                     'min_version': min_version}
+                raise exception.InvalidParam(msg)
+
         container_dict['status'] = consts.CREATING
+        extra_spec = container_dict.get('hints', None)
         new_container = objects.Container(context, **container_dict)
         new_container.create(context)
 
         if run:
-            compute_api.container_run(context, new_container)
+            compute_api.container_run(context, new_container, extra_spec,
+                                      requested_networks)
         else:
-            compute_api.container_create(context, new_container)
+            compute_api.container_create(context, new_container, extra_spec,
+                                         requested_networks)
         # Set the HTTP Location Header
         pecan.response.location = link.build_url('containers',
                                                  new_container.uuid)
         pecan.response.status = 202
         return view.format_container(pecan.request.host_url, new_container)
 
+    def _build_requested_networks(self, context, nets):
+        neutron_api = neutron.NeutronAPI(context)
+        requested_networks = []
+        for net in nets:
+            if net.get('port'):
+                port = neutron_api.get_neutron_port(net['port'])
+                neutron_api.ensure_neutron_port_usable(port)
+                requested_networks.append({'network': port['network_id'],
+                                           'port': port['id'],
+                                           'v4-fixed-ip': '',
+                                           'v6-fixed-ip': ''})
+            elif net.get('network'):
+                network = neutron_api.get_neutron_network(net['network'])
+                requested_networks.append({'network': network['id'],
+                                           'port': '',
+                                           'v4-fixed-ip': '',
+                                           'v6-fixed-ip': ''})
+
+        if not requested_networks:
+            # Find an available neutron net and create docker network by
+            # wrapping the neutron net.
+            neutron_net = neutron_api.get_available_network()
+            requested_networks.append({'network': neutron_net['id'],
+                                       'port': '',
+                                       'v4-fixed-ip': '',
+                                       'v6-fixed-ip': ''})
+
+        return requested_networks
+
+    def _check_security_group(self, context, security_group, container):
+        if security_group.get("uuid"):
+            security_group_id = security_group.get("uuid")
+            if not uuidutils.is_uuid_like(security_group_id):
+                raise exception.InvalidUUID(uuid=security_group_id)
+            if security_group_id in container.security_groups:
+                msg = _("security_group %s already present in container") % \
+                    security_group_id
+                raise exception.InvalidValue(msg)
+        else:
+            security_group_ids = utils.get_security_group_ids(
+                context, [security_group['name']])
+            if len(security_group_ids) > len(security_group):
+                msg = _("Multiple security group matches "
+                        "found for name %(name)s, use an ID "
+                        "to be more specific. ") % security_group
+                raise exception.Conflict(msg)
+            else:
+                security_group_id = security_group_ids[0]
+        container_ports_detail = utils.list_ports(context, container)
+
+        for container_port_detail in container_ports_detail:
+            if security_group_id in container_port_detail['security_groups']:
+                msg = _("security_group %s already present in container") % \
+                    list(security_group.values())[0]
+                raise exception.InvalidValue(msg)
+        return security_group_id
+
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    @validation.validated(schema.add_security_group)
+    def add_security_group(self, container_id, **security_group):
+        """Add security group to an existing container.
+
+        :param security_group: security_group to be added to container.
+        """
+
+        container = _get_container(container_id)
+        check_policy_on_container(
+            container.as_dict(), "container:add_security_group")
+        utils.validate_container_state(container, 'add_security_group')
+
+        # check if security group already presnt in container
+        context = pecan.request.context
+        compute_api = pecan.request.compute_api
+        security_group_id = self._check_security_group(
+            context, security_group, container)
+        compute_api.add_security_group(context, container,
+                                       security_group_id)
+        pecan.response.status = 202
+
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validated(schema.container_update)
@@ -296,11 +419,16 @@ class ContainersController(base.Controller):
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_delete)
-    def delete(self, container_id, force=False):
+    def delete(self, container_id, force=False, **kwargs):
         """Delete a container.
 
         :param container_ident: UUID or Name of a container.
         """
+        context = pecan.request.context
+        if is_all_tenants(kwargs):
+            policy.enforce(context, "container:delete_all_tenants",
+                           action="container:delete_all_tenants")
+            context.all_tenants = True
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:delete")
         try:
@@ -312,15 +440,13 @@ class ContainersController(base.Controller):
             utils.validate_container_state(container, 'delete')
         else:
             utils.validate_container_state(container, 'delete_force')
-        context = pecan.request.context
         compute_api = pecan.request.compute_api
         compute_api.container_delete(context, container, force)
-        container.destroy(context)
         pecan.response.status = 204
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def start(self, container_id, **kw):
+    def start(self, container_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:start")
         utils.validate_container_state(container, 'start')
@@ -334,11 +460,11 @@ class ContainersController(base.Controller):
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_stop)
-    def stop(self, container_id, timeout=None, **kw):
+    def stop(self, container_id, timeout=None, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:stop")
         utils.validate_container_state(container, 'stop')
-        LOG.debug('Calling compute.container_stop with %s' %
+        LOG.debug('Calling compute.container_stop with %s',
                   container.uuid)
         context = pecan.request.context
         compute_api = pecan.request.compute_api
@@ -348,11 +474,11 @@ class ContainersController(base.Controller):
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_reboot)
-    def reboot(self, container_id, timeout=None, **kw):
+    def reboot(self, container_id, timeout=None, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:reboot")
         utils.validate_container_state(container, 'reboot')
-        LOG.debug('Calling compute.container_reboot with %s' %
+        LOG.debug('Calling compute.container_reboot with %s',
                   container.uuid)
         context = pecan.request.context
         compute_api = pecan.request.compute_api
@@ -361,11 +487,11 @@ class ContainersController(base.Controller):
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def pause(self, container_id, **kw):
+    def pause(self, container_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:pause")
         utils.validate_container_state(container, 'pause')
-        LOG.debug('Calling compute.container_pause with %s' %
+        LOG.debug('Calling compute.container_pause with %s',
                   container.uuid)
         context = pecan.request.context
         compute_api = pecan.request.compute_api
@@ -374,11 +500,11 @@ class ContainersController(base.Controller):
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def unpause(self, container_id, **kw):
+    def unpause(self, container_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:unpause")
         utils.validate_container_state(container, 'unpause')
-        LOG.debug('Calling compute.container_unpause with %s' %
+        LOG.debug('Calling compute.container_unpause with %s',
                   container.uuid)
         context = pecan.request.context
         compute_api = pecan.request.compute_api
@@ -401,8 +527,7 @@ class ContainersController(base.Controller):
             msg = _('Valid stdout, stderr and timestamps values are ''true'', '
                     '"false", True, False, 0 and 1, yes and no')
             raise exception.InvalidValue(msg)
-        LOG.debug('Calling compute.container_logs with %s' %
-                  container.uuid)
+        LOG.debug('Calling compute.container_logs with %s', container.uuid)
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         return compute_api.container_logs(context, container, stdout, stderr,
@@ -410,7 +535,9 @@ class ContainersController(base.Controller):
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def execute(self, container_id, run=True, interactive=False, **kw):
+    @validation.validate_query_param(pecan.request,
+                                     schema.query_param_execute_command)
+    def execute(self, container_id, run=True, interactive=False, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:execute")
         utils.validate_container_state(container, 'execute')
@@ -420,18 +547,20 @@ class ContainersController(base.Controller):
         except ValueError:
             msg = _('Valid run values are true, false, 0, 1, yes and no')
             raise exception.InvalidValue(msg)
-        LOG.debug('Calling compute.container_exec with %s command %s'
-                  % (container.uuid, kw['command']))
+        LOG.debug('Calling compute.container_exec with %(uuid)s command '
+                  '%(command)s',
+                  {'uuid': container.uuid, 'command': kwargs['command']})
         context = pecan.request.context
         compute_api = pecan.request.compute_api
-        return compute_api.container_exec(context, container, kw['command'],
+        return compute_api.container_exec(context, container,
+                                          kwargs['command'],
                                           run, interactive)
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request,
                                      schema.query_param_execute_resize)
-    def execute_resize(self, container_id, exec_id, **kw):
+    def execute_resize(self, container_id, exec_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(),
                                   "container:execute_resize")
@@ -440,20 +569,23 @@ class ContainersController(base.Controller):
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         return compute_api.container_exec_resize(
-            context, container, exec_id, kw.get('h', None), kw.get('w', None))
+            context, container, exec_id, kwargs.get('h', None),
+            kwargs.get('w', None))
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validated(schema.query_param_signal)
-    def kill(self, container_id, **kw):
+    def kill(self, container_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:kill")
         utils.validate_container_state(container, 'kill')
-        LOG.debug('Calling compute.container_kill with %s signal %s'
-                  % (container.uuid, kw.get('signal', kw.get('signal'))))
+        LOG.debug('Calling compute.container_kill with %(uuid)s '
+                  'signal %(signal)s',
+                  {'uuid': container.uuid,
+                   'signal': kwargs.get('signal')})
         context = pecan.request.context
         compute_api = pecan.request.compute_api
-        compute_api.container_kill(context, container, kw.get('signal'))
+        compute_api.container_kill(context, container, kwargs.get('signal'))
         pecan.response.status = 202
 
     @pecan.expose('json')
@@ -462,8 +594,7 @@ class ContainersController(base.Controller):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:attach")
         utils.validate_container_state(container, 'attach')
-        LOG.debug('Checking the status for attach with %s' %
-                  container.uuid)
+        LOG.debug('Checking the status for attach with %s', container.uuid)
         if container.interactive:
             context = pecan.request.context
             compute_api = pecan.request.compute_api
@@ -476,15 +607,15 @@ class ContainersController(base.Controller):
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_resize)
-    def resize(self, container_id, **kw):
+    def resize(self, container_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:resize")
         utils.validate_container_state(container, 'resize')
-        LOG.debug('Calling tty resize with %s ' % (container.uuid))
+        LOG.debug('Calling tty resize with %s ', container.uuid)
         context = pecan.request.context
         compute_api = pecan.request.compute_api
-        compute_api.container_resize(context, container, kw.get('h', None),
-                                     kw.get('w', None))
+        compute_api.container_resize(context, container, kwargs.get('h', None),
+                                     kwargs.get('w', None))
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
@@ -493,35 +624,62 @@ class ContainersController(base.Controller):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:top")
         utils.validate_container_state(container, 'top')
-        LOG.debug('Calling compute.container_top with %s' %
-                  container.uuid)
+        LOG.debug('Calling compute.container_top with %s', container.uuid)
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         return compute_api.container_top(context, container, ps_args)
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def get_archive(self, container_id, **kw):
+    def get_archive(self, container_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:get_archive")
         utils.validate_container_state(container, 'get_archive')
-        LOG.debug('Calling compute.container_get_archive with %s path %s'
-                  % (container.uuid, kw['path']))
+        LOG.debug('Calling compute.container_get_archive with %(uuid)s '
+                  'path %(path)s',
+                  {'uuid': container.uuid, 'path': kwargs['path']})
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         data, stat = compute_api.container_get_archive(
-            context, container, kw['path'])
+            context, container, kwargs['path'])
         return {"data": data, "stat": stat}
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
-    def put_archive(self, container_id, **kw):
+    def put_archive(self, container_id, **kwargs):
         container = _get_container(container_id)
         check_policy_on_container(container.as_dict(), "container:put_archive")
         utils.validate_container_state(container, 'put_archive')
-        LOG.debug('Calling compute.container_put_archive with %s path %s'
-                  % (container.uuid, kw['path']))
+        LOG.debug('Calling compute.container_put_archive with %(uuid)s '
+                  'path %(path)s',
+                  {'uuid': container.uuid, 'path': kwargs['path']})
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         compute_api.container_put_archive(context, container,
-                                          kw['path'], kw['data'])
+                                          kwargs['path'], kwargs['data'])
+
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def stats(self, container_id):
+        container = _get_container(container_id)
+        check_policy_on_container(container.as_dict(), "container:stats")
+        utils.validate_container_state(container, 'stats')
+        LOG.debug('Calling compute.container_stats with %s', container.uuid)
+        context = pecan.request.context
+        compute_api = pecan.request.compute_api
+        return compute_api.container_stats(context, container)
+
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    @validation.validate_query_param(pecan.request, schema.query_param_commit)
+    def commit(self, container_id, **kwargs):
+        container = _get_container(container_id)
+        check_policy_on_container(container.as_dict(), "container:commit")
+        utils.validate_container_state(container, 'commit')
+        LOG.debug('Calling compute.container_commit %s ', container.uuid)
+        context = pecan.request.context
+        compute_api = pecan.request.compute_api
+        pecan.response.status = 202
+        return compute_api.container_commit(context, container,
+                                            kwargs.get('repository', None),
+                                            kwargs.get('tag', None))
