@@ -27,8 +27,20 @@ from zun.common.i18n import _
 #
 #             Add details of new api versions here:
 
+#
+# For each newly added microversion change, update the API version history
+# string below with a one or two line description. Also update
+# rest_api_version_history.rst for extra information on microversion.
+REST_API_VERSION_HISTORY = """REST API Version History:
+
+    * 1.1 - Initial version
+    * 1.2 - Support user specify pre created networks
+    * 1.3 - Add auto_remove to container
+    * 1.4 - Support list all container host and show a container host
+"""
+
 BASE_VER = '1.1'
-CURRENT_MAX_VER = '1.1'
+CURRENT_MAX_VER = '1.4'
 
 
 class Version(object):
