@@ -13,14 +13,13 @@
 
 import datetime
 import eventlet
+import functools
 import six
 
 from docker import errors
-from neutronclient.common import exceptions as n_exceptions
 from oslo_log import log as logging
 from oslo_utils import timeutils
 
-from zun.common import clients
 from zun.common import consts
 from zun.common import exception
 from zun.common.i18n import _
@@ -28,6 +27,7 @@ from zun.common import nova
 from zun.common import utils
 from zun.common.utils import check_container_id
 import zun.conf
+from zun.container.docker import host
 from zun.container.docker import utils as docker_utils
 from zun.container import driver
 from zun.network import network as zun_network
@@ -39,13 +39,52 @@ LOG = logging.getLogger(__name__)
 ATTACH_FLAG = "/attach/ws?logs=0&stream=1&stdin=1&stdout=1&stderr=1"
 
 
+def is_not_found(e):
+    return '404' in str(e)
+
+
+def handle_not_found(e, context, container, do_not_raise=False):
+    if container.auto_remove:
+        container.status = consts.DELETED
+    else:
+        container.status = consts.ERROR
+        container.status_reason = six.text_type(e)
+    container.save(context)
+    if do_not_raise:
+        return
+
+    raise exception.Conflict(message=_(
+        "Cannot act on container in '%s' state") % container.status)
+
+
+def wrap_docker_error(function):
+
+    @functools.wraps(function)
+    def decorated_function(*args, **kwargs):
+        context = args[1]
+        container = args[2]
+        try:
+            return function(*args, **kwargs)
+        except exception.DockerError as e:
+            if is_not_found(e):
+                handle_not_found(e, context, container)
+            raise
+
+    return decorated_function
+
+
 class DockerDriver(driver.ContainerDriver):
-    '''Implementation of container drivers for Docker.'''
+    """Implementation of container drivers for Docker."""
+    capabilities = {
+        "support_sandbox": True,
+        "support_standalone": True,
+    }
 
     def __init__(self):
         super(DockerDriver, self).__init__()
+        self._host = host.Host()
 
-    def load_image(self, image, image_path=None):
+    def load_image(self, image_path=None):
         with docker_utils.docker_client() as docker:
             if image_path:
                 with open(image_path, 'rb') as fd:
@@ -54,21 +93,36 @@ class DockerDriver(driver.ContainerDriver):
 
     def inspect_image(self, image):
         with docker_utils.docker_client() as docker:
-            LOG.debug('Inspecting image %s' % image)
-            image_dict = docker.inspect_image(image)
-            return image_dict
+            LOG.debug('Inspecting image %s', image)
+            return docker.inspect_image(image)
+
+    def get_image(self, name):
+        LOG.debug('Obtaining image %s', name)
+        with docker_utils.docker_client() as docker:
+            return docker.get_image(name)
 
     def images(self, repo, quiet=False):
         with docker_utils.docker_client() as docker:
-            response = docker.images(repo, quiet)
-            return response
+            return docker.images(repo, quiet)
 
-    def create(self, context, container, sandbox_id, image):
+    def read_tar_image(self, image):
         with docker_utils.docker_client() as docker:
+            LOG.debug('Reading local tar image %s ', image['path'])
+            tar_repo, tar_tag = docker.read_tar_image(image['path'])
+            return tar_repo, tar_tag
+
+    def create(self, context, container, image, requested_networks):
+        sandbox_id = container.get_sandbox_id()
+        network_standalone = False if sandbox_id else True
+
+        with docker_utils.docker_client() as docker:
+            network_api = zun_network.api(context=context, docker_api=docker)
             name = container.name
-            image = container.image
-            LOG.debug('Creating container with image %s name %s'
-                      % (image, name))
+            LOG.debug('Creating container with image %(image)s name %(name)s',
+                      {'image': image['image'], 'name': name})
+            if network_standalone:
+                self._provision_network(context, network_api,
+                                        requested_networks)
 
             kwargs = {
                 'name': self.get_container_name(container),
@@ -81,12 +135,15 @@ class DockerDriver(driver.ContainerDriver):
             }
 
             host_config = {}
-            host_config['network_mode'] = 'container:%s' % sandbox_id
-            # TODO(hongbin): Uncomment this after docker-py add support for
-            # container mode for pid namespace.
-            # host_config['pid_mode'] = 'container:%s' % sandbox_id
-            host_config['ipc_mode'] = 'container:%s' % sandbox_id
-            host_config['volumes_from'] = sandbox_id
+            if sandbox_id:
+                host_config['network_mode'] = 'container:%s' % sandbox_id
+                # TODO(hongbin): Uncomment this after docker-py add support for
+                # container mode for pid namespace.
+                # host_config['pid_mode'] = 'container:%s' % sandbox_id
+                host_config['ipc_mode'] = 'container:%s' % sandbox_id
+                host_config['volumes_from'] = sandbox_id
+            if container.auto_remove:
+                host_config['auto_remove'] = container.auto_remove
             if container.memory is not None:
                 host_config['mem_limit'] = container.memory
             if container.cpu is not None:
@@ -98,26 +155,73 @@ class DockerDriver(driver.ContainerDriver):
                 host_config['restart_policy'] = {'Name': name,
                                                  'MaximumRetryCount': count}
             kwargs['host_config'] = docker.create_host_config(**host_config)
-
-            response = docker.create_container(image, **kwargs)
+            image_repo = image['repo'] + ":" + image['tag']
+            response = docker.create_container(image_repo, **kwargs)
             container.container_id = response['Id']
+
+            if network_standalone:
+                addresses = self._setup_network_for_container(
+                    context, container, requested_networks, network_api)
+                container.addresses = addresses
+
             container.status = consts.CREATED
             container.status_reason = None
-            container.addresses = self.get_addresses(context, container)
             container.save(context)
             return container
 
-    def delete(self, container, force):
+    def _provision_network(self, context, network_api, requested_networks):
+        for rq_network in requested_networks:
+            self._get_or_create_docker_network(
+                context, network_api, rq_network['network'])
+
+    def _setup_network_for_container(self, context, container,
+                                     requested_networks, network_api):
+        security_group_ids = utils.get_security_group_ids(context, container.
+                                                          security_groups)
+        # Container connects to the bridge network by default so disconnect
+        # the container from it before connecting it to neutron network.
+        # This avoids potential conflict between these two networks.
+        network_api.disconnect_container_from_network(container, 'bridge')
+        addresses = {}
+        for network in requested_networks:
+            docker_net_name = self._get_docker_network_name(
+                context, network['network'])
+            addrs = network_api.connect_container_to_network(
+                container, docker_net_name, network,
+                security_groups=security_group_ids)
+            addresses[network['network']] = addrs
+
+        return addresses
+
+    def delete(self, context, container, force):
+        teardown_network = True
+        if container.get_sandbox_id():
+            teardown_network = False
+
         with docker_utils.docker_client() as docker:
+            if teardown_network:
+                network_api = zun_network.api(context=context,
+                                              docker_api=docker)
+                self._cleanup_network_for_container(context, container,
+                                                    network_api)
+
             if container.container_id:
                 try:
                     docker.remove_container(container.container_id,
                                             force=force)
                 except errors.APIError as api_error:
-                    if '404' in str(api_error):
+                    if is_not_found(api_error):
                         return
                     raise
 
+    def _cleanup_network_for_container(self, context, container, network_api):
+        if not container.addresses:
+            return
+        for neutron_net in container.addresses:
+            docker_net = self._get_docker_network_name(context, neutron_net)
+            network_api.disconnect_container_from_network(
+                container, docker_net, neutron_network_id=neutron_net)
+
     def list(self, context):
         id_to_container_map = {}
         with docker_utils.docker_client() as docker:
@@ -126,50 +230,58 @@ class DockerDriver(driver.ContainerDriver):
 
         db_containers = objects.Container.list_by_host(context, CONF.host)
         for db_container in db_containers:
+            if db_container.status in (consts.CREATING, consts.DELETED):
+                # Skip populating db record since the container is in a
+                # unstable state.
+                continue
+
             container_id = db_container.container_id
             docker_container = id_to_container_map.get(container_id)
-            if docker_container:
-                self._populate_container(db_container, docker_container)
-            else:
-                if db_container.status != consts.CREATING:
-                    # Print a warning message if the container was recorded in
-                    # DB but missing in docker.
+            if not docker_container:
+                if db_container.auto_remove:
+                    db_container.status = consts.DELETED
+                    db_container.save(context)
+                else:
                     LOG.warning("Container was recorded in DB but missing in "
                                 "docker")
+                continue
+
+            self._populate_container(db_container, docker_container)
 
         return db_containers
 
     def update_containers_states(self, context, containers):
-        my_containers = self.list(context)
-        if not my_containers:
+        db_containers = self.list(context)
+        if not db_containers:
             return
 
-        id_to_my_container_map = {container.container_id: container
-                                  for container in my_containers}
+        id_to_db_container_map = {container.container_id: container
+                                  for container in db_containers}
         id_to_container_map = {container.container_id: container
                                for container in containers}
 
         for cid in (six.viewkeys(id_to_container_map) &
-                    six.viewkeys(id_to_my_container_map)):
+                    six.viewkeys(id_to_db_container_map)):
             container = id_to_container_map[cid]
             # sync status
-            my_container = id_to_my_container_map[cid]
-            if container.status != my_container.status:
+            db_container = id_to_db_container_map[cid]
+            if container.status != db_container.status:
                 old_status = container.status
-                container.status = my_container.status
+                container.status = db_container.status
                 container.save(context)
                 LOG.info('Status of container %s changed from %s to %s',
                          container.uuid, old_status, container.status)
             # sync host
-            my_host = CONF.host
-            if container.host != my_host:
+            # Note(kiennt): Current host.
+            cur_host = CONF.host
+            if container.host != cur_host:
                 old_host = container.host
-                container.host = my_host
+                container.host = cur_host
                 container.save(context)
                 LOG.info('Host of container %s changed from %s to %s',
                          container.uuid, old_host, container.host)
 
-    def show(self, container):
+    def show(self, context, container):
         with docker_utils.docker_client() as docker:
             if container.container_id is None:
                 return container
@@ -178,9 +290,9 @@ class DockerDriver(driver.ContainerDriver):
             try:
                 response = docker.inspect_container(container.container_id)
             except errors.APIError as api_error:
-                if '404' in str(api_error):
-                    container.status = consts.ERROR
-                    container.status_reason = six.text_type(api_error)
+                if is_not_found(api_error):
+                    handle_not_found(api_error, context, container,
+                                     do_not_raise=True)
                     return container
                 raise
 
@@ -271,6 +383,13 @@ class DockerDriver(driver.ContainerDriver):
         config = response.get('Config')
         if config:
             self._populate_hostname_and_ports(container, config)
+            self._populate_command(container, config)
+
+    def _populate_command(self, container, config):
+        command_list = config.get('Cmd')
+        command_str = ' '.join(command_list)
+        if command_str:
+            container.command = command_str
 
     def _populate_hostname_and_ports(self, container, config):
         # populate hostname
@@ -285,7 +404,8 @@ class DockerDriver(driver.ContainerDriver):
         container.ports = ports
 
     @check_container_id
-    def reboot(self, container, timeout):
+    @wrap_docker_error
+    def reboot(self, context, container, timeout):
         with docker_utils.docker_client() as docker:
             if timeout:
                 docker.restart(container.container_id,
@@ -297,7 +417,8 @@ class DockerDriver(driver.ContainerDriver):
             return container
 
     @check_container_id
-    def stop(self, container, timeout):
+    @wrap_docker_error
+    def stop(self, context, container, timeout):
         with docker_utils.docker_client() as docker:
             if timeout:
                 docker.stop(container.container_id,
@@ -309,7 +430,8 @@ class DockerDriver(driver.ContainerDriver):
             return container
 
     @check_container_id
-    def start(self, container):
+    @wrap_docker_error
+    def start(self, context, container):
         with docker_utils.docker_client() as docker:
             docker.start(container.container_id)
             container.status = consts.RUNNING
@@ -317,7 +439,8 @@ class DockerDriver(driver.ContainerDriver):
             return container
 
     @check_container_id
-    def pause(self, container):
+    @wrap_docker_error
+    def pause(self, context, container):
         with docker_utils.docker_client() as docker:
             docker.pause(container.container_id)
             container.status = consts.PAUSED
@@ -325,7 +448,8 @@ class DockerDriver(driver.ContainerDriver):
             return container
 
     @check_container_id
-    def unpause(self, container):
+    @wrap_docker_error
+    def unpause(self, context, container):
         with docker_utils.docker_client() as docker:
             docker.unpause(container.container_id)
             container.status = consts.RUNNING
@@ -333,7 +457,8 @@ class DockerDriver(driver.ContainerDriver):
             return container
 
     @check_container_id
-    def show_logs(self, container, stdout=True, stderr=True,
+    @wrap_docker_error
+    def show_logs(self, context, container, stdout=True, stderr=True,
                   timestamps=False, tail='all', since=None):
         with docker_utils.docker_client() as docker:
             try:
@@ -342,25 +467,23 @@ class DockerDriver(driver.ContainerDriver):
                 tail = 'all'
 
             if since is None or since == 'None':
-                return docker.get_container_logs(container.container_id,
-                                                 stdout, stderr, False,
-                                                 timestamps, tail, None)
+                return docker.logs(container.container_id, stdout, stderr,
+                                   False, timestamps, tail, None)
             else:
                 try:
                     since = int(since)
                 except ValueError:
                     try:
-                        since = \
-                            datetime.datetime.strptime(since,
-                                                       '%Y-%m-%d %H:%M:%S,%f')
+                        since = datetime.datetime.strptime(
+                            since, '%Y-%m-%d %H:%M:%S,%f')
                     except Exception:
                         raise
-                return docker.get_container_logs(container.container_id,
-                                                 stdout, stderr, False,
-                                                 timestamps, tail, since)
+                return docker.logs(container.container_id, stdout, stderr,
+                                   False, timestamps, tail, since)
 
     @check_container_id
-    def execute_create(self, container, command, interactive=False):
+    @wrap_docker_error
+    def execute_create(self, context, container, command, interactive=False):
         stdin = True if interactive else False
         tty = True if interactive else False
         with docker_utils.docker_client() as docker:
@@ -387,32 +510,24 @@ class DockerDriver(driver.ContainerDriver):
             try:
                 docker.exec_resize(exec_id, height=height, width=width)
             except errors.APIError as api_error:
-                if '404' in str(api_error):
+                if is_not_found(api_error):
                     raise exception.Invalid(_(
                         "no such exec instance: %s") % str(api_error))
                 raise
 
     @check_container_id
-    def kill(self, container, signal=None):
+    @wrap_docker_error
+    def kill(self, context, container, signal=None):
         with docker_utils.docker_client() as docker:
             if signal is None or signal == 'None':
                 docker.kill(container.container_id)
             else:
                 docker.kill(container.container_id, signal)
-            try:
-                response = docker.inspect_container(container.container_id)
-            except errors.APIError as api_error:
-                if '404' in str(api_error):
-                    container.status = consts.ERROR
-                    container.status_reason = six.text_type(api_error)
-                    return container
-                raise
-
-            self._populate_container(container, response)
             return container
 
     @check_container_id
-    def update(self, container):
+    @wrap_docker_error
+    def update(self, context, container):
         patch = container.obj_get_changes()
 
         args = {}
@@ -428,7 +543,7 @@ class DockerDriver(driver.ContainerDriver):
             return docker.update_container(container.container_id, **args)
 
     @check_container_id
-    def get_websocket_url(self, container):
+    def get_websocket_url(self, context, container):
         version = CONF.docker.docker_remote_api_version
         remote_api_host = CONF.docker.docker_remote_api_host
         remote_api_port = CONF.docker.docker_remote_api_port
@@ -438,7 +553,8 @@ class DockerDriver(driver.ContainerDriver):
         return url
 
     @check_container_id
-    def resize(self, container, height, width):
+    @wrap_docker_error
+    def resize(self, context, container, height, width):
         with docker_utils.docker_client() as docker:
             height = int(height)
             width = int(width)
@@ -446,7 +562,8 @@ class DockerDriver(driver.ContainerDriver):
             return container
 
     @check_container_id
-    def top(self, container, ps_args=None):
+    @wrap_docker_error
+    def top(self, context, container, ps_args=None):
         with docker_utils.docker_client() as docker:
             if ps_args is None or ps_args == 'None':
                 return docker.top(container.container_id)
@@ -454,61 +571,101 @@ class DockerDriver(driver.ContainerDriver):
                 return docker.top(container.container_id, ps_args)
 
     @check_container_id
-    def get_archive(self, container, path):
+    @wrap_docker_error
+    def get_archive(self, context, container, path):
         with docker_utils.docker_client() as docker:
             stream, stat = docker.get_archive(container.container_id, path)
             filedata = stream.read()
             return filedata, stat
 
     @check_container_id
-    def put_archive(self, container, path, data):
+    @wrap_docker_error
+    def put_archive(self, context, container, path, data):
         with docker_utils.docker_client() as docker:
             docker.put_archive(container.container_id, path, data)
 
+    @check_container_id
+    @wrap_docker_error
+    def stats(self, context, container):
+        with docker_utils.docker_client() as docker:
+            res = docker.stats(container.container_id, decode=False,
+                               stream=False)
+
+            cpu_usage = res['cpu_stats']['cpu_usage']['total_usage']
+            system_cpu_usage = res['cpu_stats']['system_cpu_usage']
+            cpu_percent = float(cpu_usage) / float(system_cpu_usage) * 100
+            mem_usage = res['memory_stats']['usage'] / 1024 / 1024
+            mem_limit = res['memory_stats']['limit'] / 1024 / 1024
+            mem_percent = float(mem_usage) / float(mem_limit) * 100
+
+            blk_stats = res['blkio_stats']['io_service_bytes_recursive']
+            io_read = 0
+            io_write = 0
+            for item in blk_stats:
+                if 'Read' == item['op']:
+                    io_read = io_read + item['value']
+                if 'Write' == item['op']:
+                    io_write = io_write + item['value']
+
+            net_stats = res['networks']
+            net_rxb = 0
+            net_txb = 0
+            for k, v in net_stats.items():
+                net_rxb = net_rxb + v['rx_bytes']
+                net_txb = net_txb + v['tx_bytes']
+
+            stats = {"CONTAINER": container.name,
+                     "CPU %": cpu_percent,
+                     "MEM USAGE(MiB)": mem_usage,
+                     "MEM LIMIT(MiB)": mem_limit,
+                     "MEM %": mem_percent,
+                     "BLOCK I/O(B)": str(io_read) + "/" + str(io_write),
+                     "NET I/O(B)": str(net_rxb) + "/" + str(net_txb)}
+            return stats
+
+    @check_container_id
+    @wrap_docker_error
+    def commit(self, context, container, repository=None, tag=None):
+        with docker_utils.docker_client() as docker:
+            repository = str(repository)
+            if tag is None or tag == "None":
+                return docker.commit(container.container_id, repository)
+            else:
+                tag = str(tag)
+                return docker.commit(container.container_id, repository, tag)
+
     def _encode_utf8(self, value):
-        if six.PY2 and not isinstance(value, unicode):
-            value = unicode(value)
+        if six.PY2 and not isinstance(value, six.text_type):
+            value = six.text_type(value)
         return value.encode('utf-8')
 
-    def create_sandbox(self, context, container, image='kubernetes/pause',
-                       networks=None):
+    def create_sandbox(self, context, container, requested_networks,
+                       image='kubernetes/pause'):
         with docker_utils.docker_client() as docker:
             network_api = zun_network.api(context=context, docker_api=docker)
-            if networks is None:
-                # Find an available neutron net and create docker network by
-                # wrapping the neutron net.
-                neutron_net = self._get_available_network(context)
-                network = self._get_or_create_docker_network(
-                    context, network_api, neutron_net['id'])
-                networks = [network['Name']]
-
+            self._provision_network(context, network_api, requested_networks)
             name = self.get_sandbox_name(container)
             sandbox = docker.create_container(image, name=name,
                                               hostname=name[:63])
-            # Container connects to the bridge network by default so disconnect
-            # the container from it before connecting it to neutron network.
-            # This avoids potential conflict between these two networks.
-            network_api.disconnect_container_from_network(sandbox, 'bridge')
-            for network in networks:
-                network_api.connect_container_to_network(sandbox, network)
+            container.set_sandbox_id(sandbox['Id'])
+            addresses = self._setup_network_for_container(
+                context, container, requested_networks, network_api)
+            if addresses is None:
+                raise exception.ZunException(_(
+                    "Unexpected missing of addresses"))
+
+            container.addresses = addresses
+            container.save(context)
+
             docker.start(sandbox['Id'])
             return sandbox['Id']
 
-    def _get_available_network(self, context):
-        neutron = clients.OpenStackClients(context).neutron()
-        search_opts = {'tenant_id': context.project_id, 'shared': False}
-        nets = neutron.list_networks(**search_opts).get('networks', [])
-        if not nets:
-            raise exception.ZunException(_(
-                "There is no neutron network available"))
-        nets.sort(key=lambda x: x['created_at'])
-        return nets[0]
-
     def _get_or_create_docker_network(self, context, network_api,
                                       neutron_net_id):
         # Append project_id to the network name to avoid name collision
         # across projects.
-        docker_net_name = neutron_net_id + '-' + context.project_id
+        docker_net_name = self._get_docker_network_name(context,
+                                                        neutron_net_id)
         docker_networks = network_api.list_networks(names=[docker_net_name])
         if not docker_networks:
             network_api.create_network(neutron_net_id=neutron_net_id,
@@ -516,18 +673,21 @@ class DockerDriver(driver.ContainerDriver):
             docker_networks = network_api.list_networks(
                 names=[docker_net_name])
 
-        return docker_networks[0]
+    def _get_docker_network_name(self, context, neutron_net_id):
+        # Append project_id to the network name to avoid name collision
+        # across projects.
+        return neutron_net_id + '-' + context.project_id
 
-    def delete_sandbox(self, context, sandbox_id):
+    def delete_sandbox(self, context, container):
+        sandbox_id = container.get_sandbox_id()
         with docker_utils.docker_client() as docker:
             network_api = zun_network.api(context=context, docker_api=docker)
-            sandbox = docker.inspect_container(sandbox_id)
-            for network in sandbox["NetworkSettings"]["Networks"]:
-                network_api.disconnect_container_from_network(sandbox, network)
+            self._cleanup_network_for_container(context, container,
+                                                network_api)
             try:
                 docker.remove_container(sandbox_id, force=True)
             except errors.APIError as api_error:
-                if '404' in str(api_error):
+                if is_not_found(api_error):
                     return
                 raise
 
@@ -535,65 +695,12 @@ class DockerDriver(driver.ContainerDriver):
         with docker_utils.docker_client() as docker:
             docker.stop(sandbox_id)
 
-    def get_sandbox_id(self, container):
-        if container.meta:
-            return container.meta.get('sandbox_id', None)
-        else:
-            LOG.warning("Unexpected missing of sandbox_id")
-            return None
-
-    def set_sandbox_id(self, container, sandbox_id):
-        if container.meta is None:
-            container.meta = {'sandbox_id': sandbox_id}
-        else:
-            container.meta['sandbox_id'] = sandbox_id
-
     def get_sandbox_name(self, container):
         return 'zun-sandbox-' + container.uuid
 
     def get_container_name(self, container):
         return 'zun-' + container.uuid
 
-    def get_addresses(self, context, container):
-        neutron = clients.OpenStackClients(context).neutron()
-        sandbox_id = self.get_sandbox_id(container)
-        with docker_utils.docker_client() as docker:
-            addresses = {}
-            response = docker.inspect_container(sandbox_id)
-            networks = response["NetworkSettings"]["Networks"]
-            for name, network in networks.items():
-                neutron_net_id = name.rsplit('-', 1)[0]
-                if not neutron_net_id:
-                    continue
-
-                try:
-                    neutron_net = neutron.show_network(neutron_net_id)
-                    neutron_net_name = neutron_net['network'].get('name')
-                except n_exceptions.NetworkNotFoundClient:
-                    neutron_net_name = ''
-                if not neutron_net_name:
-                    continue
-
-                v4_address = network.get("IPAddress", "")
-                v6_address = network.get("GlobalIPv6Address", "")
-                mac_address = network["MacAddress"]
-                ports = neutron.list_ports(mac_address=mac_address)['ports']
-                port_id = ports[0]['id'] if ports else ''
-                addresses[neutron_net_name] = [
-                    {
-                        'addr': v4_address,
-                        'version': 4,
-                        'port': port_id
-                    },
-                    {
-                        'addr': v6_address,
-                        'version': 6,
-                        'port': port_id
-                    },
-                ]
-
-            return addresses
-
     def get_host_info(self):
         with docker_utils.docker_client() as docker:
             info = docker.info()
@@ -606,8 +713,15 @@ class DockerDriver(driver.ContainerDriver):
             os_type = info['OSType']
             os = info['OperatingSystem']
             kernel_version = info['KernelVersion']
+            labels = {}
+            slabels = info['Labels']
+            if slabels:
+                for l in slabels:
+                    kv = l.split("=")
+                    label = {kv[0]: kv[1]}
+                    labels.update(label)
             return (total, running, paused, stopped, cpus,
-                    architecture, os_type, os, kernel_version)
+                    architecture, os_type, os, kernel_version, labels)
 
     def get_cpu_used(self):
         cpu_used = 0
@@ -618,17 +732,38 @@ class DockerDriver(driver.ContainerDriver):
                 # Fixme: if there is a way to get all container inspect info
                 # for one call only?
                 inspect = docker.inspect_container(cnt_id)
-                nanocpus = inspect['HostConfig']['NanoCpus']
                 cpu_period = inspect['HostConfig']['CpuPeriod']
                 cpu_quota = inspect['HostConfig']['CpuQuota']
                 if cpu_period and cpu_quota:
                     cpu_used += float(cpu_quota) / cpu_period
                 else:
-                    cpu_used += float(nanocpus) / 1e9
+                    if 'NanoCpus' in inspect['HostConfig']:
+                        nanocpus = inspect['HostConfig']['NanoCpus']
+                        cpu_used += float(nanocpus) / 1e9
             return cpu_used
 
+    def add_security_group(self, context, container, security_group):
+
+        with docker_utils.docker_client() as docker:
+            network_api = zun_network.api(context=context,
+                                          docker_api=docker)
+            network_api.add_security_groups_to_ports(container,
+                                                     [security_group])
+
+    def get_available_nodes(self):
+        return [self._host.get_hostname()]
+
 
 class NovaDockerDriver(DockerDriver):
+    capabilities = {
+        "support_sandbox": True,
+        "support_standalone": False,
+    }
+
+    def add_security_group(self, context, container, security_group, **kwargs):
+        msg = "NovaDockerDriver does not support security_groups"
+        raise exception.ZunException(msg)
+
     def create_sandbox(self, context, container, key_name=None,
                        flavor='m1.tiny', image='kubernetes/pause',
                        nics='auto'):
@@ -655,7 +790,7 @@ class NovaDockerDriver(DockerDriver):
         return sandbox_id
 
     def _ensure_active(self, novaclient, server, timeout=300):
-        '''Wait until the Nova instance to become active.'''
+        """Wait until the Nova instance to become active."""
         def _check_active():
             return novaclient.check_active(server)
 
@@ -672,8 +807,7 @@ class NovaDockerDriver(DockerDriver):
         novaclient = nova.NovaClient(elevated)
         server_name = self._find_server_by_container_id(sandbox_id)
         if not server_name:
-            LOG.warning("Cannot find server name for sandbox %s" %
-                        sandbox_id)
+            LOG.warning("Cannot find server name for sandbox %s", sandbox_id)
             return
 
         server_id = novaclient.delete_server(server_name)
@@ -684,13 +818,12 @@ class NovaDockerDriver(DockerDriver):
         novaclient = nova.NovaClient(elevated)
         server_name = self._find_server_by_container_id(sandbox_id)
         if not server_name:
-            LOG.warning("Cannot find server name for sandbox %s" %
-                        sandbox_id)
+            LOG.warning("Cannot find server name for sandbox %s", sandbox_id)
             return
         novaclient.stop_server(server_name)
 
     def _ensure_deleted(self, novaclient, server_id, timeout=300):
-        '''Wait until the Nova instance to be deleted.'''
+        """Wait until the Nova instance to be deleted."""
         def _check_delete_complete():
             return novaclient.check_delete_server_complete(server_id)
 
@@ -705,7 +838,7 @@ class NovaDockerDriver(DockerDriver):
     def get_addresses(self, context, container):
         elevated = context.elevated()
         novaclient = nova.NovaClient(elevated)
-        sandbox_id = self.get_sandbox_id(container)
+        sandbox_id = container.get_sandbox_id()
         if sandbox_id:
             server_name = self._find_server_by_container_id(sandbox_id)
             if server_name:
